/*-
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: cdriver.c,v 1.1.1.1 2002/09/25 18:11:29 ggv Exp $
 *
 * Contents: Main procedure for invoking the PDNET FORTRAN 
 *   subroutines for the solution of linear minimum cost network flow 
 *   problems.
 *
 *   Inputs a network with pdrdmc(), allocates work array and calls 
 * pdnet() to compute minimum cost network flow, deals with error 
 * conditions  with pdierr(), and prints output report with pdwrsl().
 *
 * Authors: Luis Portugal (Univ. Coimbra - Portugal)
 *          Mauricio G. C. Resende (AT&T Labs Research - USA)
 *          Geraldo Veiga (AT&T Labs - USA)
 *          Joaquim Judice (Univ. Coimbra - Portugal)
 *          Joao Patricio (EST Tomar - Portugal)
 *
 * Reference: L.F. Portugal, M.G.C. Resende, G. Veiga, and
 * J.J. Judice, "A truncated primal-infeasible dual-feasible
 * interior point network flow method," Networks, vol. 35,
 * pp. 91-108, 2000.
 *
 *--------------------------------------------------------------------
 */
#include "pdnet.h"		/* PDNET general definitions */
#include "debug.h"		/* PDNET debug macros */

int
main (int argc, char *argv[])
{
  long nn = 0,			/* number of nodes */
    na = 0,			/* number of arcs */
    nwkd = 0,			/* size of double work array */
   *strn = (long *) NULL,	/* start node of arc */
   *endn = (long *) NULL,	/* end node of arc */
   *b = (long *) NULL,		/* array of capacities at each node */
   *c = (long *) NULL,		/* array of costs at each arc */
   *l = (long *) NULL,		/* lower bound of flow at each arc */
   *u = (long *) NULL,		/* upper bound of flow at each arc */
   *optflo = (long *) NULL,	/* array with optimal flow at each arc */
    rc;				/* function return code */

  long
    info[30],		/* array with parameters & statistics */
   *ptrwki = (long *) NULL;	/* alias pointer to work space */

  double dinfo[35],		/* array with parameters & statistics */
   *ptrwkd = (double *) NULL;	/* pointer to work space */

/*--------------------------------------------------------------------
 * Runtime parameters 
 *--------------------------------------------------------------------
 */
  long iprcnd,			/* preconditioner used */
    maxa,			/* maximum number of arcs */
    maxit,			/* maximum primal-dual iterations */
    maxn,			/* maximum number of nodes */
    maxwkd,			/* maximum size of double work array */
    mxcgit,			/* maximum number of PCG iterations */
    optgap, 			/* duality gap optimality */
    optmf, 			/* maximum flow optimality */
    optst, 			/* spanning tree optimality */
    prttyp;			/* printing level */


/*--------------------------------------------------------------------
 * Read the problem data. This C-language procedure reads the 
 * problem instance in standard DIMACS forma from the standard input.
 * Memory is allocate and the network data structure returned via the
 * function arguments.
 *--------------------------------------------------------------------
 */
  (void) pdrdmm (&nn, &na, &strn, &endn, &b, &c, &l, &u);
  maxa = na;
  maxn = nn;

/*--------------------------------------------------------------------
 *   Allocate workspace and solution arrays.
 *--------------------------------------------------------------------
 */
  (void) pdwrkd_ (&na, &nn, &nwkd);
  maxwkd = nwkd;
  MALLOC (ptrwkd, double, maxwkd);
  MALLOC (ptrwki, long, maxwkd);
  ptrwki = (long *) ptrwkd; 
  MALLOC (optflo, long, maxa);

/*--------------------------------------------------------------------
 * Set run time parameters.
 *--------------------------------------------------------------------
 */
  iprcnd = 3;
  maxit = 1000;
  prttyp = 3;
  mxcgit = 1000;
  optgap = 1;
  optmf = 1;
  optst = 1;
  /*  (void) pdsprm_ (dinfo, info, &iprcnd, &maxit, &optgap, &optmf, &optst, 
      &prttyp, &mxcgit); */

  (void) pdsprm_ (info, &maxa, &maxn);

/*--------------------------------------------------------------------
 * Solve the problem problem instance.
 *--------------------------------------------------------------------
 */
  rc = pdnet_ (b, c, dinfo, endn, info, l, &maxa, &maxn, &maxwkd, &na, &nn,
	       &nwkd, optflo, ptrwkd, ptrwki, strn, u);

/* ------------------------------------------------------------------
 *    Write output report.
 * ------------------------------------------------------------------ 
 */
  if (rc == 0)
    {
      (void) pdwrsl_ (c, dinfo, endn, info, l, &na, &nn, strn, u, optflo);
    }
 
  return (rc);
}
