c     $Id: fdriver.f,v 1.1.1.1 2002/09/25 18:11:29 ggv Exp $

      program pddrvr

c     ------------------------------------------------------------------      
c     driver:  Driver program for pdnet.f: Fortran subroutines for the
c              solution of linear minimum cost network flow problems.
c
c              Inputs a network with pdrdmc(), calls pdnet() to compute
c              minimum cost network flow, deals with error conditions 
c              with pdierr(), and prints output report with pdwrsl().
c
c     Authors: Luis Portugal (Univ. Coimbra - Portugal)
c              Mauricio G. C. Resende (AT&T Labs Research - USA)
c              Geraldo Veiga (AT&T Labs - USA)
c              Joaquim Judice (Univ. Coimbra - Portugal)
c              Joao Patricio (EST Tomar - Portugal)
c
c     Reference: L.F. Portugal, M.G.C. Resende, G. Veiga, and 
c     J.J. Judice, "A truncated primal-infeasible dual-feasible 
c     interior point network flow method," Networks, vol. 35, 
c     pp. 91-108, 2000.
c     
c     ------------------------------------------------------------------
      implicit none
      
      equivalence (ptrwkd, ptrwki)

c     ------------------------------------------------------------------    
c     Maximum dimensions arrays.
c
c               maxa   : maximum number of arcs.
c               maxn   : maximum number of nodes.
c               maxwrk : maximum size for working space.
c     ------------------------------------------------------------------ 
      integer maxa  , maxn  , maxwrk, pdnet, rc, i, nwkd
      parameter (maxa   =  200000)
      parameter (maxn   =   10000)  
      parameter (maxwrk = 4000000)

c     ------------------------------------------------------------------
c     Integer arrays:
c
c               b     (1:maxn): array of capacities at each node.
c               c     (1:maxa): array of costs at each arc.
c               endn  (1:maxa): end vertex of edge.
c               info  (1:  30): some integer statistics.
c               l     (1:maxa): lower bound of flow at each arc.
c               optflo(1:maxa): array with optimal flow at each arc.
c               strn  (1:maxa): start vertex of edge.
c               u     (1:maxa): upper bound of flow at each arc.
c     ------------------------------------------------------------------
      integer  
     +        b     (1:maxn),
     +        c     (1:maxa), 
     +        endn  (1:maxa),  
     +        info  (1:  30),
     +        l     (1:maxa), 
     +        optflo(1:maxa),
     +        ptrwki(1:maxwrk),
     +        strn  (1:maxa), 
     +        u     (1:maxa)

c     ------------------------------------------------------------------
c     Integer variables:
c
c                na: number of arcs.
c                nn: number of nodes.
c     ------------------------------------------------------------------
      integer     na,      nn, iprcnd,  maxit, mxcgit, optgap,  optmf,
     +         optst,  prttyp

c     ------------------------------------------------------------------
c     Double precision arrays:
c
c         dinfo  (1:    35): array with parameters & statistics.
c         ptrwkd(1:maxwrk): double precision pointer to work space
c     ------------------------------------------------------------------
      double precision 
     +                 dinfo (1:    35),
     +                 ptrwkd(1:maxwrk)

c     ------------------------------------------------------------------
c     Start of executable portion of main program.
c
c     Read the problem data. This C-language procedure reads the 
c     problem in standard DIMACS format. The network has nn nodes and
c     na arcs.  Nodes of arc(i) are placed in strn(i) and endn(i).
c     The cost, upper bound, and lower bound of arc(i) are placed in 
c     c(i), u(i), and l(i), respectively.  The supply/demand of node(i) 
c     is placed in b(i).
c     ------------------------------------------------------------------
      call pdrdmc(nn, na, strn, endn, b, c, l, u, maxn, maxa)

c     ------------------------------------------------------------------      
c     Startup definitions.  These paramters are placed in array info.
c     
c     Required info parameters for input are:
c
c          info(23) - maximum number of arcs
c          info(24) - maximum number of nodes
c          info( 3) = fortran output unit
c          info( 2) = selected preconditioner 
c          info(19) = output level
c          info( 3) = unit of output file (6=standard output)
c     ------------------------------------------------------------------
c
c     ------------------------------------------------------------------
c     Set preconditioner to be used.
c
c     info( 2) = -1 : diagonal preconditioner.
c     info( 2) = -2 : IQRD preconditioner.
c     info( 2) = -3 : Spanning tree preconditioner.
c     info( 2) = -4 : diagonally compensated spanning tree 
c                     preconditioner.
c     ------------------------------------------------------------------
c
c     ------------------------------------------------------------------
c     Set output level.
c
c     info(19) =  0 : completely silent.
c     info(19) =  1 : only prints iteration progression.
c     info(19) =  2 : also prints some statistics at each iteration.
c     info(19) =  3 : same as previous, but also prints solution.
c     ------------------------------------------------------------------
c
c     ------------------------------------------------------------------
c     Solve the problem.  If optimal solution is found, optimal flow 
c     on arc(i) is returned in optflo(i).  The error condition is in
c     info(50).  On return, array info contains statistics in addition
c     to parameters:
c
c           dinfo( 1): dual objective function value
c           dinfo( 2): dual objective function value on tree
c           dinfo( 5): p-d gap
c           dinfo(10): miu
c            info(10): p-d iterations
c           dinfo(15): primal objective function
c           dinfo(16): primal objective function value on tree
c            info(15): error condition
c     ------------------------------------------------------------------
      iprcnd = 3
      maxit  = 1000
      prttyp = 1
      mxcgit = 1000
      optgap = 1
      optmf  = 1
      optst  = 1

      call pdsprm (info,  maxa, maxn)
      
      rc =   pdnet(b, c, dinfo, endn, info, l, maxa, maxn, maxwrk, 
     +             na, nn, nwkd, optflo, ptrwkd, ptrwki, strn , u ) 

c     ------------------------------------------------------------------
c     Deal with error conditions.  Subroutine pdierr prints out
c     the error condition.
c     ------------------------------------------------------------------
      if (info(15) .ne. 0) then
         call pdierr(info(15))
         stop
      endif

c     ------------------------------------------------------------------
c     Write output report.
c     ------------------------------------------------------------------
      call pdwrsl( c, dinfo, endn, info, l, na, nn, strn, u, optflo) 

c     ------------------------------------------------------------------
c     End of main program.
c     ------------------------------------------------------------------
      stop                                                           
      end
