
 





      subroutine pdchmf( av4,    cap,    cost,  endv,  mfstat, mxfe,
     +                   mxfv,   nbuk,   nedge,  nvrtx,  optdl,  rc,
     +                   ssol,   stedg,  stptol, stptr,  stpval, strv,
     +                   stvrtx, supply, tolhi,  tollo,  wrk2e1, wrkdv1,
     +                   wrke1,  wrke2,  wrke3,  wrke4,  wsol,   
     +                   xopt,   xsol,   ysol,   zsol )
c     ------------------------------------------------------------------
c     pdchmf: This subroutine checks optimality of flow by using max
c             flow criterion:
c
c             1. Identify active, upper bound and lower bound edges.
c
c             2. Compute spanning tree (maximal forest) for active edges.
c
c             3. Build tentative dual optimal solution.
c
c             4. Identify dual face.
c
c             5. Solve max flow for edges defining optimal face.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             mxfe  : edges on optimal flow network.
c             mxfv  : vertices on optimal flow network.
c             nbuk  : number of buckets.
c             nedge : number of arcs.
c             nvrtx : number of nodes.
c             rc    : flags optimality status.
c     ------------------------------------------------------------------
      integer mxfe  , mxfv  , nbuk  , nedge , nvrtx , rc

c     ------------------------------------------------------------------
c     Integer arrays:
c
c             endv  (1:  nedge): end vertex in network data structure
c             stedg (1:  nedge): edges on spanning tree.
c             stptr (1:  nedge): pointer for edges on spanning tree.
c             stvrtx(1:  nedge): vertices on the spanning forest.
c             strv  (1:  nedge): start vertex in network data structure.
c             wrk2e1(1:2*nedge): working array of order 2*nedge.
c             wrke1 (1:  nedge): working array of order nedge.
c             wrke2 (1:  nedge): working array of order nedge.
c             wrke3 (1:  nedge): working array of order nedge.
c             wrke4 (1:  nedge): working array of order nedge.
c             xopt  (1:  nedge): optimal solution of maxflow problem.
c     ------------------------------------------------------------------
      integer
     +        endv  (1:  nedge),
     +        stedg (1:  nedge),
     +        stptr (1:  nedge),
     +        stvrtx(1:  nedge),
     +        strv  (1:  nedge),
     +        wrk2e1(1:2*nedge),
     +        wrke1 (1:  nedge),
     +        wrke2 (1:  nedge),
     +        wrke3 (1:  nedge),
     +        wrke4 (1:  nedge),
     +        xopt  (1:  nedge)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c              av4   (1:nedge): auxiliary vector.
c              cap   (1:nedge): capacity at each arc.
c              cost (1:nvrtx): cost at each node.
c              ssol  (1:nedge): solution for s.
c              supply(1:nvrtx): supply at each node.
c              wrkdv1(1:nvrtx): work array. 
c              wsol  (1:nedge): solution for w.
c              xsol  (1:nedge): solution for x.
c              ysol  (1:nvrtx): solution for y.
c              zsol  (1:nedge): solution for z.
c     ------------------------------------------------------------------
      double precision
     +                 av4   (1:nedge),
     +                 cap   (1:nedge),
     +                 cost (1:nvrtx),
     +                 ssol  (1:nedge),
     +                 supply(1:nvrtx),
     +                 wrkdv1(1:nvrtx),
     +                 wsol  (1:nedge),
     +                 xsol  (1:nedge),
     +                 ysol  (1:nvrtx),
     +                 zsol  (1:nedge)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             i     : loop counter.
c             mflow : maximum flow supplied.
c             mfreq : maximum flow requested.
c             mfstat: optimality status.
c             nact  : number of active edges.
c             nactmf: number of edges in active list.
c             ncap  : number of edges at the upper bound.
c             ncapmf: number of edges at capacity.
c             ntree : number of trees in spanning forest.
c     ------------------------------------------------------------------
      integer i     , mflow , mfreq , mfstat, nact  , nactmf, ncap  ,
     +        ncapmf, ntree

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c             optdl : dual objective value.
c             stptol: zero for dual slacks.
c             stpval: largest slack on dual face.
c             tolhi : upper bound of the tolerance for maxflow.
c             tollo : lower bound of the tolerance for maxflow.
c     ------------------------------------------------------------------
      double precision optdl , stptol, stpval, tolhi , tollo

c     ------------------------------------------------------------------
c     Double precision variables:
c
c             pddlob: function that computes dual objective value.
c     ------------------------------------------------------------------
      double precision pddlob

c     ------------------------------------------------------------------
c     External functions
c
c             pddlob: function that computes dual objective value.
c     ------------------------------------------------------------------
      external pddlob

c     ------------------------------------------------------------------
c     Select active edges.
c     ------------------------------------------------------------------
      call pdtpia( wrke1,  wrke2,  nact,   ncap,   nedge,  ssol,
     +             tolhi,  tollo,  wsol,   xsol,   zsol )

c     ------------------------------------------------------------------
c     Sort active edges for spanning tree.
c     ------------------------------------------------------------------
      do 20 i=1,nedge
         av4(i)=zsol(i)/xsol(i) + wsol(i)/ssol(i)
 20   continue
      call pdsedg( wrke1,  wrke4,  nact,   nbuk,   wrke2,  wrke3,
     +             av4 )

c     ------------------------------------------------------------------
c     Identify spanning tree for active edges.
c     ------------------------------------------------------------------
      call pdmxst( endv,   nact,   ntree,  nvrtx,  wrke2,  wrke3,
     +             wrke4, stedg,    strv )

c     ------------------------------------------------------------------
c     Build data structure for spanning forest.
c     ------------------------------------------------------------------
      call pdbdst( ntree,  nvrtx,  wrke3,  stptr,  stvrtx )

c     ------------------------------------------------------------------
c     Order spanning forest.
c     ------------------------------------------------------------------
      call pdodst( endv,  wrk2e1, wrke3,  nedge,  ntree,  nvrtx,
     +             wrke4,  stedg,  stptr,  strv, stvrtx )

c     ------------------------------------------------------------------
c     Build tentative dual optimal.
c     ------------------------------------------------------------------
      call pddlot( cost,  endv,  nedge,  ntree,  nvrtx,  stedg,
     +             stptr,  strv, stvrtx, wrkdv1, ysol )

c     ------------------------------------------------------------------
c     Identify optimal dual face.
c     ------------------------------------------------------------------
      call pddlof( wrke1,  cost,   endv,  stpval, nactmf, ncapmf,
     +             nedge,  nvrtx,   strv,  stptol, wrkdv1 )

c     ------------------------------------------------------------------
c     Compute dual objective value.
c     ------------------------------------------------------------------
      optdl = pddlob(     cap,  cost,  endv,  nedge,  nvrtx,  supply, 
     +                   strv, wrkdv1 )

c     ------------------------------------------------------------------
c     Solve maxflo problem.
c     ------------------------------------------------------------------
      call pdmxfl( wrke1,  cap,      endv,  mflow,  mfreq,  mxfe,
     +             mxfv,   nactmf, ncapmf,  nedge,  nvrtx,  strv, 
     +             supply,   xopt )

c     ------------------------------------------------------------------
c     Return optimality status.
c     ------------------------------------------------------------------
      if(mfreq.eq.mflow) then
         rc=0
         mfstat=2
      else
         rc=1
         mfstat=1
      endif
      return

c     ------------------------------------------------------------------
c     End of subroutine pdchmf.
c     ------------------------------------------------------------------
      end
