/*-
 * $Id: pderr.c,v 1.1.1.1 2002/09/25 18:11:29 ggv Exp $
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Contents: Error message printing procedures
 *
 * Authors: Luis Portugal (Univ. Coimbra - Portugal)
 *          Mauricio G. C. Resende (AT&T Labs Research - USA)
 *          Geraldo Veiga (AT&T Labs - USA)
 *          Joaquim Judice (Univ. Coimbra - Portugal)
 *          Joao Patricio (EST Tomar - Portugal)
 *
 * User callable functions:
 *
 * >>> syserr <<<
 * >>> usrerr <<<
 * >>> usrmsg <<<
 * >>> eprintf <<<
 *
 * Global variables:
 *
 * pderr_name - Program name to be imported into calling routine
 *
 * Revision History: 
 *   2001-12-27	Geraldo Veiga
 *       Initial release for ACM distribution.
 *
 */

#include <stdio.h>
#include <errno.h>
#include <stdarg.h>

char *pderr_name;		/* Program name to be imported into calling routine */
extern int errno;		/* Number of last error from errno.h */

/*-
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * >>> syserr <<<
 *
 * Synopsis: void syserr (const char *format,...)
 *
 * Description: print a formatted message and exit.  If errno is
 * non-zero, it  will use perror to print the corresponding system
 * error message. Lastly, Syserr will terminate execution with an exit
 * code of 1.
 *
 * Return Value: none
 *
 * Side Effects: All output is written to stderr. Program execution
 * is terminated.
 *
 * -------------------------------------------------------------------
 */

void
syserr (char *format, ...)
{
  int save_err;
  va_list ap;

/* Save error code */
  save_err = errno;

/* Print program name */
  if (pderr_name && *pderr_name)
    fprintf (stderr, "%s: ", pderr_name);
  else
    fprintf (stderr, "***** ");

/* Print message */
  va_start (ap, format);
  (void) vfprintf (stderr, format, ap);
  va_end (ap);

  fputc ('\n', stderr);

/* Print message and error code */
  if (save_err)
    {
      errno = save_err;
      perror ("System error");
    }
  fflush (stderr);
  exit (1);
}

/*-
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * >>> usrerr <<<
 *
 * Synopsis: void usrerr (const char *format,...)
 *
 * Description: print a formatted message and exit with code 1.
 *
 * Return Value: none
 *
 * Side Effects: All output is written to stderr. Program execution
 * is terminated.
 *
 * -------------------------------------------------------------------
 */

void
usrerr (char *format, ...)
{
  va_list ap;

/* Print program name */
  if (pderr_name && *pderr_name)
    fprintf (stderr, "%s: ", pderr_name);
  else
    fprintf (stderr, "***** ");

/* Print message */
  va_start (ap, format);
  (void) vfprintf (stderr, format, ap);
  va_end (ap);

  fputc ('\n', stderr);

  fflush (stderr);

  exit (1);
}

/*-
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * >>> usrmsg <<<
 *
 * Synopsis: void usrmsg (const char *format,...)
 *
 * Description: print a formatted message and return.
 *
 * Return Value: none
 *
 * Side Effects: All output is written to stderr. Program execution
 * is terminated.
 *
 * -------------------------------------------------------------------
 */

void
usrmsg (char *format, ...)
{
  va_list ap;

/* Print program name */
  if (pderr_name && *pderr_name)
    fprintf (stderr, "%s: ", pderr_name);
  else
    fprintf (stderr, "***** ");

/* Print message */
  va_start (ap, format);
  (void) vfprintf (stderr, format, ap);
  va_end (ap);

  fputc ('\n', stderr);

  fflush (stderr);

}

/*-
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * >>> eprintf <<<
 *
 * Synopsis: void eprintf (const char *format,...)
 *
 * Description: Prints a formatted message on stderr.  It behaves
 * exactly like printf with the sole exception being that it writes to
 * stderr instead of stdout.
 *
 * Return Value: none
 *
 * Side Effects: All output is written to stderr.
 *
 * -------------------------------------------------------------------
 */

int
eprintf (char *format, ...)
{
  int rc;
  va_list ap;

  va_start (ap, format);
  rc = vfprintf (stderr, format, ap);
  va_end (ap);

  fflush (stderr);
  return rc;
}
