







      subroutine pdodst(  endv, edglst, edgptr, nedge,  ntree,  nvrtx,
     +                   stack,  stedg,  stptr,  strv, stvrtx )
c     ------------------------------------------------------------------
c     pdodst: Given a spanning tree or forest, this subroutine finds
c             permutations for vertices and tree edges that yield an
c             incidence matrix with a block triangular structure.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             nedge : number of edges.
c             ntree : number of trees.
c             nvrtx : number of vertices.
c     ------------------------------------------------------------------
      integer nedge , ntree , nvrtx

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             endv  (1:  nedge): end vertex in network data structure
c             edglst(1:2 * (nvrtx - ntree): list for edges in tree adjacent
c                                           to each vertex.
c             edgptr(1:    nvrtx + 1): pointer to beginning edge list for 
c                                      each vertex.
c             stack (1:    nvrtx    ): stack of eligible vertices in dfs
c             stedg (1:    nedge    ): edge linking vertex to parent in
c                                      a rooted tree representation.
c             stptr (1:    ntree + 1): pointer to first vertex in
c                                      a tree.
c             strv  (1:  nedge): start vertex in network data structure.
c             stvrtx(1:    nvrtx    ): list of vertices in spanning
c                                      forest. Returns a triagular
c                                      ordering of vertices.
c     ------------------------------------------------------------------
      integer
     +        endv (1: nedge    ),
     +        edglst(1:2 * nvrtx -1),
     +        edgptr(1:    nvrtx + 1),
     +        stack (1:    nvrtx    ),
     +        stedg (1:    nedge    ),
     +        stptr (1:    ntree + 1),
     +        str v (1: nedge    ),
     +        stvrtx(1:    nvrtx    )

c     ------------------------------------------------------------------
c     Integer variables:
c
c             edg   : edge index.
c             eptr  : temporary pointer to the st-edge data structure.
c             ntedge: number of edges in spanning forest.
c             nxtvtx: adjacent vertex.
c             tedg  : index for tree edges.
c             top   : poiter to top of stack.
c             tree  : tree counter.
c             vptr  : temporary pointer to the vertex data structure.
c             vrtx  : vertex index.
c     ------------------------------------------------------------------
      integer edg   , eptr  , ntedge, nxtvtx, tedg  , top   , tree  ,
     +        vptr  , vrtx

c     ------------------------------------------------------------------
c     Initialize vertex degrees in edgptr array.  This array will be
c     used as pointers to the edglst array, indicating the beginning of 
c     incoming and outgoing edge list  for each vertex.  
c     ------------------------------------------------------------------
      do 10 vrtx=1,nvrtx
         edgptr(vrtx)=0
 10   continue

c     ------------------------------------------------------------------
c     Compute degree of each edge.
c     ------------------------------------------------------------------
      ntedge=nvrtx-ntree
      do 20 tedg=1,ntedge
         edg=stedg(tedg)
         vrtx=strv(edg)
         edgptr(vrtx)=edgptr(vrtx)+1
         vrtx=endv(edg)
         edgptr(vrtx)=edgptr(vrtx)+1
 20   continue

c     ------------------------------------------------------------------
c     Set edge list pointers after the end of list (+1) for each vertex.
c     ------------------------------------------------------------------
      edgptr(1)=edgptr(1)+1
      do 30 vrtx=2,nvrtx
         edgptr(vrtx)=edgptr(vrtx)+edgptr(vrtx-1)
 30   continue

c     ------------------------------------------------------------------
c     Build lists of edges using stored pointers.  After this operation,
c     the pointers indicate the beginning of the edge list for each 
c     vertex.
c     ------------------------------------------------------------------
      do 40 tedg=ntedge,1,-1
         edg=stedg(tedg)
         vrtx=strv(edg)
         edgptr(vrtx)=edgptr(vrtx)-1
         edglst(edgptr(vrtx))=edg
         vrtx=endv(edg)
         edgptr(vrtx)=edgptr(vrtx)-1
         edglst(edgptr(vrtx))=edg
 40   continue
      edgptr(nvrtx+1)=2*ntedge+1

c     ------------------------------------------------------------------
c     Initialize parent edges.
c     ------------------------------------------------------------------
      do 50 vrtx=1,nvrtx
         stedg(vrtx)=0
 50   continue

c     ------------------------------------------------------------------
c     Traverse each tree in depth first order saving dfs rank and
c     parent edge.
c     ------------------------------------------------------------------
      do 60 tree=1,ntree
c        ---------------------------------------------------------------
c        Initialize current vertex with tree root (parent edge 0).
c        ---------------------------------------------------------------
         vptr=stptr(tree)
         vrtx=stvrtx(vptr)

c        ---------------------------------------------------------------
c        Initialize stack of eligible vertices.
c        ---------------------------------------------------------------
         top=1
         stack(top)=vrtx

c        ---------------------------------------------------------------
c        Position pointer to first edge in list for vrtx.
c        ---------------------------------------------------------------
         eptr=edgptr(vrtx)

c        ---------------------------------------------------------------
c        Start Do until with empty.stack
c        ---------------------------------------------------------------
 70      continue

c        ---------------------------------------------------------------
c        Test for end of edge list.
c        ---------------------------------------------------------------
         if(eptr.ge.edgptr(vrtx+1)) then
            top=top-1

c           ------------------------------------------------------------
c           Skip to next tree if stack is empty.
c           ------------------------------------------------------------
            if(top.le.0) go to 60

c           ------------------------------------------------------------
c           Identify vertex and saved pointer to edge list.
c           ------------------------------------------------------------
            vrtx=stack(top)
            eptr=edglst(edgptr(vrtx))
            go to 70
            end if

c           ------------------------------------------------------------
c           Identify next successor edge.
c           ------------------------------------------------------------
            edg=edglst(eptr)

c           ------------------------------------------------------------
c           Advance and save edge pointer for current vertex.
c           ------------------------------------------------------------
            eptr=eptr+1
            edglst(edgptr(vrtx))=eptr

c           ------------------------------------------------------------
c           Skip if edge is parent.
c           ------------------------------------------------------------
            if(edg.eq.stedg(vrtx)) go to 70

c           ------------------------------------------------------------
c           Identify next vertex (edg=(vrtx,nxtvtx) in rooted tree).
c           ------------------------------------------------------------
            nxtvtx=strv(edg)
            if(nxtvtx.eq.vrtx) nxtvtx=endv(edg)

c           ------------------------------------------------------------
c           Save dfs order and parent edge.
c           ------------------------------------------------------------
            vptr=vptr+1
            stvrtx(vptr)=nxtvtx
            stedg(nxtvtx)=edg

c           ------------------------------------------------------------
c           Add visited vertex to stack of eligible vertices.
c           ------------------------------------------------------------
            top=top+1
            vrtx=nxtvtx
            stack(top)=vrtx

c           ------------------------------------------------------------
c           Position pointer to first edge in list for vrtx.
c           ------------------------------------------------------------
            eptr=edgptr(vrtx)
            go to 70

c        ---------------------------------------------------------------
c        End of "do until".
c        ---------------------------------------------------------------
 60   continue
      return

c     ------------------------------------------------------------------
c     End of subroutine pdodst.
c     ------------------------------------------------------------------
      end
